import { useEffect, useRef, useState } from "react";
import { readImageDicomFileSeries } from "@itk-wasm/dicom";
import vtkITKHelper from "@kitware/vtk.js/Common/DataModel/ITKHelper";
import vtkVolume from "@kitware/vtk.js/Rendering/Core/Volume";
import vtkFullScreenRenderWindow from "@kitware/vtk.js/Rendering/Misc/FullScreenRenderWindow";
import "@kitware/vtk.js/Rendering/Profiles/Volume";
import vtkRenderer from "@kitware/vtk.js/Rendering/Core/Renderer";
import vtkVolumeMapper from "@kitware/vtk.js/Rendering/Core/VolumeMapper";
import vtkRenderWindow from "@kitware/vtk.js/Rendering/Core/RenderWindow";

import "./app.css";
import {
  checkExistenceInIndexedDB,
  loadImageFromIndexedDB,
  saveImageToIndexedDB,
} from "./util";
import { extractAxialSliceAsImage } from "./extract";

function App() {
  const [files, setFiles] = useState<File[]>([]);
  const inputRef = useRef<HTMLInputElement>(null);

  const axialRef = useRef<HTMLElement | null>(null);
  const sagittalRef = useRef<HTMLElement | null>(null);
  const coronalRef = useRef<HTMLElement | null>(null);

  const rendererRef = useRef<vtkRenderer>();
  const mapperRef = useRef<vtkVolumeMapper>();
  const actorRef = useRef<vtkVolume>();
  const renderWindowRef = useRef<vtkRenderWindow>();

  const [image, setImage] = useState();

  useEffect(() => {
    if (inputRef.current) inputRef.current.webkitdirectory = true;
  }, []);

  useEffect(() => {
    checkExistenceInIndexedDB("outputImageKey").then((exist) => {
      if (exist) {
        console.time("start load from cache");
        loadImageFromIndexedDB().then((outputImage) => {
          const fullScreenRenderer = vtkFullScreenRenderWindow.newInstance({
            container: axialRef.current,
          });
          rendererRef.current = fullScreenRenderer.getRenderer();
          renderWindowRef.current = fullScreenRenderer.getRenderWindow();
          actorRef.current = vtkVolume.newInstance();
          mapperRef.current = vtkVolumeMapper.newInstance({
            sampleDistance: 1.1,
          });
          const vtkImage = vtkITKHelper.convertItkToVtkImage(outputImage);
          mapperRef.current?.setInputData(vtkImage);
          mapperRef.current && actorRef.current?.setMapper(mapperRef.current);
          actorRef.current && rendererRef.current?.addVolume(actorRef.current);
          rendererRef.current?.resetCamera();
          renderWindowRef.current?.render();
          console.timeEnd("start load from cache");

          const imgSource = extractAxialSliceAsImage(vtkImage, 1);
          setImage(imgSource);
        });
      }
    });
  }, []);

  useEffect(() => {
    if (files.length > 0) {
      const fullScreenRenderer = vtkFullScreenRenderWindow.newInstance({
        container: axialRef.current,
      });
      rendererRef.current = fullScreenRenderer.getRenderer();
      renderWindowRef.current = fullScreenRenderer.getRenderWindow();
      actorRef.current = vtkVolume.newInstance();
      mapperRef.current = vtkVolumeMapper.newInstance({ sampleDistance: 1.1 });
      console.time("itk-wasm parse vtkVolumeImageData");
      readImageDicomFileSeries(null, {
        inputImages: files,
      }).then((image) => {
        const vtkImage = vtkITKHelper.convertItkToVtkImage(image.outputImage);
        console.timeEnd("itk-wasm parse vtkVolumeImageData");
        saveImageToIndexedDB(image.outputImage);
        mapperRef.current?.setInputData(vtkImage);
        mapperRef.current && actorRef.current?.setMapper(mapperRef.current);
        actorRef.current && rendererRef.current?.addVolume(actorRef.current);
        rendererRef.current?.resetCamera();
        renderWindowRef.current?.render();
      });

      return () => {
        actorRef.current = undefined;
        mapperRef.current = undefined;
        rendererRef.current = undefined;
        renderWindowRef.current = undefined;
      };
    }
  }, [files]);

  const handleFolderUpload = async (
    event: React.ChangeEvent<HTMLInputElement>
  ) => {
    const target = event.target as HTMLInputElement;
    if (!target.files) return;
    const files = [...target.files].filter((file) =>
      file.name.endsWith(".dcm")
    );
    setFiles(files);
  };

  return (
    <>
      <input
        type="file"
        onChange={handleFolderUpload}
        ref={inputRef}
        style={{ position: "fixed", zIndex: 100 }}
      />
      <div className="mpr">
        <div ref={axialRef} style={{ width: 500, height: 500 }}></div>
        {/* <div ref={sagittalRef} style={{ width: 300, height: 300 }}></div>
        <div ref={coronalRef} style={{ width: 300, height: 300 }}></div> */}
      </div>
      <img src={image} style={{ width: 200, height: 200 }} />
    </>
  );
}

export default App;
